
#include "auth.h"
#include "debug.h"
#include "domain_name_cache.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>

static int auth_hostname_assert( struct link *link, time_t stoptime )
{
	char line[AUTH_LINE_MAX];

	if(!link_readline(link,line,sizeof(line),stoptime)) {
		debug("auth_hostname: lost connection");
		return 0;
	}

	if(!strcmp(line,"yes")) {
		debug("auth_hostname: accepted");
		return 1;
	}

	return 0;
}

static int auth_hostname_accept( struct link *link, char **subject, time_t stoptime )
{
	char addr[LINK_ADDRESS_MAX];
	char name[DOMAIN_NAME_MAX];
	int port;

	if(!link_address_remote(link,addr,&port)) {
		debug("auth_hostname: couldn't get address of link");
		goto reject;
	}

	if(!domain_name_cache_lookup_reverse(addr,name)) {
		debug("auth_hostname: couldn't look up name of %s",name);
		goto reject;
	}

	*subject = strdup(name);
	if(!*subject) {
		debug("auth_hostname: out of memory");
		goto reject;
	}

	link_write(link,"yes\n",4,stoptime);
	return 1;

	reject:
	link_write(link,"no\n",3,stoptime);
	return 0;
}

int auth_hostname_register()
{
	debug("auth_hostname: registered");
	return auth_register( "hostname", auth_hostname_assert, auth_hostname_accept );
}

