/*
dttools is Copyright (C) 2002 Douglas Thain
This software is released under the GNU General Public License.
See the file COPYING for details.
*/


#include "interval_set.h"
#include "macros.h"

#include <stdlib.h>
#include <stdio.h>

#define ISLF INTERVAL_SET_LENGTH_FORMAT

struct interval {
	interval_set_length_t position;
	interval_set_length_t length;
	struct interval *next;
};

struct interval * interval_create( interval_set_length_t position, interval_set_length_t length, struct interval *next )
{
	struct interval *i;

	i = malloc(sizeof(*i));
	if(!i) return 0;

	i->position = position;
	i->length = length;
	i->next = next;

	return i;
}

struct interval_set {
	struct interval *head;
};

struct interval_set * interval_set_create()
{
	struct interval_set *s;

	s = malloc(sizeof(*s));
	if(!s) return 0;

	s->head = 0;
	return s;
}

void interval_set_delete( struct interval_set *s )
{
	if(s) {
		while(s->head) {
			struct interval *i = s->head->next;
			free(s->head);
			s->head = i;
		}
		free(s);
	}
}

int  interval_set_load( struct interval_set *s, FILE *file )
{
	interval_set_length_t position, length;

	while(fscanf(file,ISLF " " ISLF,&position,&length)==2) {
		if(!interval_set_add(s,position,length)) return 0;
	}

	return 1;
}

int interval_set_save( struct interval_set *s, FILE *file )
{
	struct interval *i;

	for(i=s->head;i;i=i->next) {
		if(fprintf(file,ISLF " " ISLF "\n",i->position,i->length)<=0) return 0;
	}

	return 1;
}

static int overlaps( struct interval *i, interval_set_length_t position, interval_set_length_t length )
{
	interval_set_length_t end = position+length;
	interval_set_length_t i_end = i->position+i->length;

	if( (position>=i->position) && (position<=i_end) ) return 1;
	if( (end>=i->position) && (end<i_end) ) return 1;
	if( (position<i->position) && (end>=i_end) ) return 1;

	return 0;
}

static void combine( struct interval *i, interval_set_length_t position, interval_set_length_t length )
{
	interval_set_length_t end = position+length;
	interval_set_length_t i_end = i->position+i->length;

	if(position<i->position) i->position=position;
	if(end>i_end) {
		i->length = end-i->position;
	} else {
		i->length = i_end-i->position;
	}
}


int interval_set_add( struct interval_set *s, interval_set_length_t position, interval_set_length_t length )
{
	struct interval *i;

	for( i=s->head; i; i=i->next ) {
		if(overlaps(i,position,length)) {
			combine(i,position,length);
			return 1;
		}
	}

	i = interval_create(position,length,s->head);
	if(i) {
		s->head = i;
		return 1;
	} else {
		return 0;
	}

	return 0;
}

int interval_set_contains( struct interval_set *s, interval_set_length_t position, interval_set_length_t length, interval_set_length_t *actual )
{
	struct interval *i;

	for(i=s->head;i;i=i->next) {
		if( position>=i->position && position<(i->position+i->length) ) {
			*actual = MIN(length,i->length-(position-i->position));
			return 1;
		}
	}

	return 0;
}

void interval_set_nbytes( struct interval_set *s, interval_set_length_t *size )
{
	struct interval *i;
	*size = 0;

	for(i=s->head;i;i=i->next) {
		*size += i->length;
	}
}

