/*
dttools is Copyright (C) 2002 Douglas Thain
This software is released under the GNU General Public License.
See the file COPYING for details.
*/


#include "sound.h"
#include "synth.h"
#include "int_sizes.h"

#include <math.h>
#include <stdlib.h>
#include <stdio.h>

#ifndef PI
#define PI 3.141592654
#endif

struct synth {
	INT16_T *data;
	int samples;
	int frequency;
	int sample_rate;
};

struct synth * synth_create( int sample_rate, int frequency, int length )
{
	struct synth *s;
	int falloff_width;
	int amplitude = 1000;
	int i;

	s = malloc(sizeof(struct synth));
	if(!s) return 0;

	s->sample_rate = sample_rate;
	s->samples = sample_rate*length/1000;
	falloff_width = s->samples/10;

	s->data = malloc(s->samples*sizeof(*s->data));
	if(!s->data) {
		free(s);
		return 0;
	}

	for(i=0;i<s->samples;i++) {
		s->data[i] = amplitude*sin( 2*PI*frequency*i/s->samples );
	}

	for(i=0;i<falloff_width;i++) {
		s->data[i] = s->data[i]*i/falloff_width;
	}

	for(i=0;i<s->samples;i++) {
		s->data[s->samples-i] = s->data[s->samples-i]*i/falloff_width;
	}

	return s;
}

int synth_play( struct synth *s )
{
	int audio;
	struct sound_config c;

	c.encoding = SOUND_ENCODING_LINEAR;
	c.bits_per_sample = 8*sizeof(*s->data);
	c.samples_per_second = s->sample_rate;

	audio = sound_open(&c);
	if(audio<0) return 0;

	sound_play(audio,s->data,s->samples);
	sound_close(audio);

	return 1;
}

void synth_delete( struct synth *s )
{
	if(!s) return;
	free(s->data);
	free(s);
}


